package ru.ldeloff.servermonitorbot.service.ssh;

import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.objects.Update;
import ru.ldeloff.servermonitorbot.model.SshServer;
import ru.ldeloff.servermonitorbot.repository.SshRepository;

import java.io.ByteArrayOutputStream;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class SshServiceImpl implements  SshService {
    final SshRepository sshRepository;
    @Override
    public SendMessage getStatusSessions(Update update) {
        SendMessage message = new SendMessage();
        long chatId = update.getMessage().getChatId();
        message.setText(sshRepository.getStatusSessions());
        message.setChatId(chatId);
        return message;
    }
    @Override
    public List<SshServer> getSshServers() {
        return sshRepository.getSshServers();
    }

    @Override
    public String execute(String s, SshServer sshServer) {
        return  execute(s, sshServer, 100);
    }

    @Override
    public String execute(String s, SshServer sshServer, int timeout) {
        Session session = sshServer.getSession();
        ChannelExec channel = null;
        try {
            channel = (ChannelExec) session.openChannel("exec");
            channel.setCommand(s);
            ByteArrayOutputStream responseStream = new ByteArrayOutputStream();
            channel.setOutputStream(responseStream);
            channel.connect();

            while (channel.isConnected()) {
                Thread.sleep(timeout);
            }

            String responseString = new String(responseStream.toByteArray());
            return responseString;
        } catch (JSchException | InterruptedException e) {
            log.error(e.getMessage());
            return null;
        } finally {
            if (channel != null) {
                channel.disconnect();
            }
        }
    }
}
