package ru.ldeloff.servermonitorbot.service.uname;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.objects.Update;
import ru.ldeloff.servermonitorbot.model.SshServer;
import ru.ldeloff.servermonitorbot.service.ssh.SshService;
import ru.ldeloff.servermonitorbot.utils.ui.uname.UnameChatButtonAggregate;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Slf4j
public class UnameServiceImpl implements UnameService {
    final SshService sshService;
    final UnameChatButtonAggregate unameChatButtonAggregate;
    @Override
    public SendMessage sendUnameAggregate(Update update) {
        SendMessage message = new SendMessage();
        long chatId = update.getMessage().getChatId();
        message.setChatId(chatId);
        return unameChatButtonAggregate.uiForm(message);
    }
    @Override
    public SendMessage uname(Update update) {
        String messageText = update.getCallbackQuery().getData();

        String [] tags = Arrays.stream(messageText.split(":"))
                .map(String::trim)
                .toArray(String[]::new);

        String result;
        if (tags[1].equals("all")) {
            result = unameAllHost();
        } else {
            result = unameSpecificHost(tags[1]);
        }
        SendMessage message = new SendMessage();
        long chatId = update.getCallbackQuery().getMessage().getChatId();
        message.setChatId(chatId);
        message.setText(result);
        return message;
    }

    private String unameAllHost() {
       List<SshServer> servers = sshService.getSshServers();
       StringBuilder response = new StringBuilder();
       servers.forEach(server -> {
           response.append(unameSpecificHost(server.getName())).append("\n");
       });
       return response.toString();
    }

    private String unameSpecificHost(String serverName) {
        Optional<SshServer> server = sshService.getSshServers()
                .stream()
                .filter(x -> x.getName().equals(serverName))
                .findFirst();
        if (server.isPresent()) {
            String result = sshService.execute("uname -a", server.get());
            return server.get().getName() + ": " + (Objects.isNull(result) ? "ошибка при выполнении команды" : result);

        } else {
            log.error("Ошибка при выполнении команды 'uname'. Искомый сервер (" + serverName + ") не найден");
            return null;
        }
    }
}
